# Useful function for the fast merge
Recode <- function(var, from, to){
x <- as.vector(var)
x.tmp <- x
for (i in 1:length(from)){x <- replace(x, x.tmp == from[i], to[i])}
if(is.factor(var)) factor(x) else x}

# LOAD SPECIES LIST
all <- read.table("lg.txt", sep="\t", as.is=T, strip.white=T)

# LOAD "FILTERS"
# Family names for every genus
families <- read.table("filters/families.txt", sep="\t", as.is=T, strip.white=T)
# Table of synonyms
synonyms <- read.table("filters/synonyms.txt", sep="\t", as.is=T, strip.white=T)
# Species authorship
auth <- read.table("filters/species_authorship.txt", sep="\t", as.is=T, strip.white=T, quote="")

# CLEAN ALL SPECIES LISTS
all$V1 <- gsub(" +", " ", all$V1) # remove double spaces
if (length(grep(" sp[.p]*$", all$V1)) != 0) all <- all[-grep(" sp[.p]*$", all$V1),] # remove all lines with "sp.", "spp." etc.
all$V1 <- Recode(all$V1, synonyms$V1, synonyms$V2) # replace synonyms with valid names
all <- all[all$V1 != "" & !is.na(all$V1),] # remove "empty" species (appeared after replacing unspecified synonyms) 

# ADD FAMILIES
genera1 <- gsub(" [_a-z-]+$", "", all$V1) # separate genera names
families1 <- Recode(genera1, families$V2, families$V1)
families2 <- ifelse(genera1 %in% families$V2, families1, "")

# ===
# HTML
# ===

# This section may be created if html output is needed

# ===
# TeX
# ===

# SORT LIST BY FAMILIES
all4 <- all[order(families2, all$V1),] # sort by family then by species

# MAKE SECTIONS FOR FAMILIES
FAM2 <- sort(families2)
tmp <- FAM2[1]
for (i in 2:(length(FAM2)))
{
if (FAM2[i]==tmp)
	{
	FAM2[i] <- ""
	}
else
	{
	tmp <- FAM2[i]
	}
}	
FAM <- ifelse(FAM2!="", paste("\\FAMILY{", FAM2, "}", sep=""), "")

# MAKE SPECIES ENTRIES
SPC1 <- ifelse(Recode(all4$V1, auth$V1, auth$V2) == all4$V1, "", paste(all4$V1, Recode(all4$V1, auth$V1, auth$V2), sep=" ")) # insert authorship
SPC2 <- gsub("&", "\\\\D\\\\", SPC1)
SPC3 <- gsub("x_", "\\\\(\\\\times\\\\)", SPC2)
SPC <- ifelse(SPC3 != "", paste("\\SPECIES{", SPC3, "}", sep=""), "") # surround with \SPECIES command

# MAKE COMMENTS
CMM <- ifelse(all4$V4 != "", paste("\\COMMENT{", all4$V4, "}", sep=""), "") # surround with \COMMENT command

write.table(file="list_body.tex", data.frame(FAM, SPC, CMM), quote=F, row.names=F, col.names=F, sep=" ", eol="\n\n")
